#include "geosim.h"
#include "gspriv.h"
#include "ietypes.h"
#include "vwinds.h"
#include "windfunc.h"
/*#include "rootw.h"*/
#include "iemisc.h"
#include "ifw.h"
#include "objprop.h"
/*	
	The window handled by this source file displays
	each window along with the window's viewable 
	objects.
*/


extern boolean windpop;
extern IEIconfig iestate;
extern GSColor redCol, blackCol;

static GSsclistHandle windScroll;
static GSlistHandle windList;
static int numWind, curItem, numPop;
static boolean changed;

/* Internal protos */
void SetUpVWindsList();
IEWindow *GetWindowNum(int num);
void wTextFunc(GSlistHandle l, int slot);
void wInFunc();
void wOutFunc();
void wClickFunc(int slot);
char *wLabelFunc(int slot);
/**/

void wTextFunc(GSlistHandle l, int slot)
{
	if (slot/2 == curItem)
		GSsetcolor(redCol);
	else
		GSsetcolor(blackCol);
}

void wInFunc()
{
}

void wOutFunc()
{
}

void wClickFunc(int slot)
{
	IEWindow *win;
	IEPopup *pop;
	int old = curItem;
	
	curItem = slot/2;
	
	GSdrawlistitem(windList, New, 2*old);
	GSdrawlistitem(windList, New, 2*old+1);

	if ((slot%2)==1) {
		if (slot/2 < numWind) {
			win = GetWindowNum(slot/2);
			win->visibleineditor = !win->visibleineditor;
			}
		else {
			pop = GetPopupNum(slot/2 - numWind);
			pop->visibleineditor = !pop->visibleineditor;
			}
		changed = TRUE;
		}
		
	GSdrawlistitem(windList, New, 2*curItem);
	GSdrawlistitem(windList, New, 2*curItem+1);

	if (old == NO_ITEM)
		GSenablebutton("vwinds", "EditProp", TRUE);
}

char *wLabelFunc(int slot)
{
	IEWindow *win;
	IEPopup *pop;
	
	if ((slot/2) < numWind) {
		win = GetWindowNum(slot/2);
		if (win) {
			if ((slot%2)==0)
				return win->name;
			else
				return win->visibleineditor?
						"visible" : "invisible";
			}
		}
	else if ((slot/2)-numWind < numPop) {
		pop = GetPopupNum(slot/2 - numWind);
		if (pop) {
			if ((slot%2)==0)
				return pop->title;
			else
				return pop->visibleineditor?
						"visible" : "invisible";
			}
		}
		
	return "ERROR! End of Window List!";
}

void SetUpVWindsList()
{
	IEWindow *curw = iestate.winlisthead;
	IEPopup *curp = iestate.popuplisthead;
	int x, y, wd, ht;
	
	numWind = numPop = 0;
	curItem = NO_ITEM;
	
	while (curw != NULL) {
		numWind++;
		curw = curw->nextwin;
		}
	while (curp != NULL) {
		numPop++;
		curp = curp->nextpopup;
		}
		
	GSsclistsetup("vwinds", "windpick",
					2,
					"vwinditem",
					0, 0,
					JUST_LEFT,
					wLabelFunc,
					wTextFunc,
					wInFunc,
					wOutFunc,
					wClickFunc,
					"windscroll",
					GSconvertcolor(THUMBCOL),
					GSconvertcolor(THUMBTL),
					GSconvertcolor(THUMBBR),
					2*(numWind+numPop), 0,
					NULL,
					"wup", "wdown",
					&windList, &windScroll);
	
	GSdrawsclist(windScroll);

	GSgetdragareaspecs("vwinds", "windpick", 
				&x, &y, &wd, &ht, NULL, NULL);
	ShadowRect(x, y, wd, ht, THIN, Concave);	
}

void DrawVWindsPopup()
{
	windpop = TRUE;
	changed = FALSE;
	GSdisablebutton("vwinds", "EditProp", FALSE);
	GSdrawnamedwindow("vwinds", POPUP, BSTORE);
	SetUpVWindsList();
}

void RedrawVWindsPopup()
{
	int x, y, wd, ht;

	GSredrawnamedwindow("vwinds");
	GSdrawsclist(windScroll);

	GSgetdragareaspecs("vwinds", "windpick", 
				&x, &y, &wd, &ht, NULL, NULL);
	ShadowRect(x, y, wd, ht, THIN, Concave);	
}

void RemoveVWindsPopup(boolean redraw)
{
	windpop = FALSE;
	GSremovenamedwindow("vwinds");
	GSdismantlesclist(windScroll);
	if (changed && redraw)
		DrawInterface(TRUE);
}

boolean vwind_pick(int x, int y, DragStatusType dr)
{
	GShandlelist(windList, x, y, dr);
}

boolean vwind_scroll(int x, int y, DragStatusType dr)
{
	GShandlesclist(windScroll, x, y, dr);
}

boolean vwind_up()
{
	GShandleupbutton(windScroll);
}

boolean vwind_down()
{
	GShandledownbutton(windScroll);
}

boolean vwind_ok()
{
	RemoveVWindsPopup(TRUE);
}

boolean vwind_edit_prop()
{
	if (curItem == NO_ITEM) {
		GSdisablebutton("vwinds", "EditProp", TRUE);
		return;			 /* shouldn't be here anyway */
		}
		
	RemoveVWindsPopup(FALSE);
	if (curItem < numWind)
		DrawWPropWind(GetWindowNum(curItem), FALSE);
	else if (curItem-numWind < numPop) {
		PopupToProp(GetPopupNum(curItem-numWind));
		DrawPropWind(FALSE);
		}
}

IEWindow *GetWindowNum(int num)
{
	IEWindow *cur = iestate.winlisthead;
	
	if (num < 0)
		return NULL;
		
	while (cur != NULL) {
		if (num == 0)
			return cur;
		num--;
		cur = cur->nextwin;
		}
	
	return NULL;
}